@@ -27,7 +27,7 @@ Module for communication with gvmd in `Greenbone Management Protocol version 9`_
 import collections
 import numbers
 
-from typing import Any, List, Optional, Callable
+from typing import Any, List, Optional, Callable, Tuple
 
 from gvm.errors import InvalidArgument, InvalidArgumentType, RequiredArgument
 from gvm.utils import deprecation
@@ -367,6 +367,90 @@ class GmpV9Mixin(GvmProtocol):
             function=self.create_config.__name__,
         )
 
+    def create_permission(
+        self,
+        name: str,
+        subject_id: str,
+        subject_type: PermissionSubjectType,
+        *,
+        resource_id: Optional[str] = None,
+        resource_type: Optional[EntityType] = None,
+        comment: Optional[str] = None
+    ) -> Any:
+        """Create a new permission
+
+        Arguments:
+            name: Name of the new permission
+            subject_id: UUID of subject to whom the permission is granted
+            subject_type: Type of the subject user, group or role
+            comment: Comment for the permission
+            resource_id: UUID of entity to which the permission applies
+            resource_type: Type of the resource. For Super permissions user,
+                group or role
+
+        Returns:
+            The response. See :py:meth:`send_command` for details.
+        """
+        if not name:
+            raise RequiredArgument(
+                function=self.create_permission.__name__, argument='name'
+            )
+
+        if not subject_id:
+            raise RequiredArgument(
+                function=self.create_permission.__name__, argument='subject_id'
+            )
+
+        if not isinstance(subject_type, PermissionSubjectType):
+            raise InvalidArgumentType(
+                function=self.create_permission.__name__,
+                argument='subject_type',
+                arg_type=PermissionSubjectType.__name__,
+            )
+
+        cmd = XmlCommand("create_permission")
+        cmd.add_element("name", name)
+
+        _xmlsubject = cmd.add_element("subject", attrs={"id": subject_id})
+        _xmlsubject.add_element("type", subject_type.value)
+
+        if comment:
+            cmd.add_element("comment", comment)
+
+        if resource_id or resource_type:
+            if not resource_id:
+                raise RequiredArgument(
+                    function=self.create_permission.__name__,
+                    argument='resource_id',
+                )
+
+            if not resource_type:
+                raise RequiredArgument(
+                    function=self.create_permission.__name__,
+                    argument='resource_type',
+                )
+
+            if not isinstance(resource_type, self.types.EntityType):
+                raise InvalidArgumentType(
+                    function=self.create_permission.__name__,
+                    argument='resource_type',
+                    arg_type=self.types.EntityType.__name__,
+                )
+
+            _xmlresource = cmd.add_element(
+                "resource", attrs={"id": resource_id}
+            )
+
+            _actual_resource_type = resource_type
+            if resource_type.value == EntityType.AUDIT.value:
+                _actual_resource_type = EntityType.TASK
+            elif resource_type.value == EntityType.POLICY.value:
+                _actual_resource_type = EntityType.SCAN_CONFIG
+
+            _xmlresource.add_element("type", _actual_resource_type.value)
+
+        return self._send_xml_command(cmd)
+
     def create_policy(
         self, name: str, *, policy_id: str = None, comment: Optional[str] = None
     ) -> Any:
@@ -391,6 +475,92 @@ class GmpV9Mixin(GvmProtocol):
             function=self.create_policy.__name__,
         )
 
+    def create_tag(
+        self,
+        name: str,
+        resource_type: EntityType,
+        *,
+        resource_filter: Optional[str] = None,
+        resource_ids: Optional[List[str]] = None,
+        value: Optional[str] = None,
+        comment: Optional[str] = None,
+        active: Optional[bool] = None
+    ) -> Any:
+        """Create a tag.
+
+        Arguments:
+            name: Name of the tag. A full tag name consisting of namespace and
+                predicate e.g. `foo:bar`.
+            resource_type: Entity type the tag is to be attached to.
+            resource_filter: Filter term to select resources the tag is to be
+                attached to. Only one of resource_filter or resource_ids can be
+                provided.
+            resource_ids: IDs of the resources the tag is to be attached to.
+                Only one of resource_filter or resource_ids can be provided.
+            value: Value associated with the tag.
+            comment: Comment for the tag.
+            active: Whether the tag should be active.
+
+        Returns:
+            The response. See :py:meth:`send_command` for details.
+        """
+        if not name:
+            raise RequiredArgument(
+                function=self.create_tag.__name__, argument='name'
+            )
+
+        if resource_filter and resource_ids:
+            raise InvalidArgument(
+                "create_tag accepts either resource_filter or resource_ids "
+                "argument",
+                function=self.create_tag.__name__,
+            )
+
+        if not resource_type:
+            raise RequiredArgument(
+                function=self.create_tag.__name__, argument='resource_type'
+            )
+
+        if not isinstance(resource_type, self.types.EntityType):
+            raise InvalidArgumentType(
+                function=self.create_tag.__name__,
+                argument='resource_type',
+                arg_type=EntityType.__name__,
+            )
+
+        cmd = XmlCommand('create_tag')
+        cmd.add_element('name', name)
+
+        _xmlresources = cmd.add_element("resources")
+        if resource_filter is not None:
+            _xmlresources.set_attribute("filter", resource_filter)
+
+        for resource_id in resource_ids or []:
+            _xmlresources.add_element(
+                "resource", attrs={"id": str(resource_id)}
+            )
+
+        _actual_resource_type = resource_type
+        if resource_type.value == EntityType.AUDIT.value:
+            _actual_resource_type = EntityType.TASK
+        elif resource_type.value == EntityType.POLICY.value:
+            _actual_resource_type = EntityType.SCAN_CONFIG
+        _xmlresources.add_element("type", _actual_resource_type.value)
+
+        if comment:
+            cmd.add_element("comment", comment)
+
+        if value:
+            cmd.add_element("value", value)
+
+        if active is not None:
+            if active:
+                cmd.add_element("active", "1")
+            else:
+                cmd.add_element("active", "0")
+
+        return self._send_xml_command(cmd)
+
     def create_task(
         self,
         name: str,
@@ -488,6 +658,169 @@ class GmpV9Mixin(GvmProtocol):
 
         return self._send_xml_command(cmd)
 
+    def get_aggregates(
+        self,
+        resource_type: EntityType,
+        *,
+        filter: Optional[str] = None,
+        filter_id: Optional[str] = None,
+        sort_criteria: Optional[list] = None,
+        data_columns: Optional[list] = None,
+        group_column: Optional[str] = None,
+        subgroup_column: Optional[str] = None,
+        text_columns: Optional[list] = None,
+        first_group: Optional[int] = None,
+        max_groups: Optional[int] = None,
+        mode: Optional[int] = None,
+        **kwargs
+    ) -> Any:
+        """Request aggregated information on a resource / entity type
+
+        Additional arguments can be set via the kwargs parameter for backward
+        compatibility with older versions of python-gvm, but are not validated.
+
+        Arguments:
+            resource_type: The entity type to gather data from
+            filter: Filter term to use for the query
+            filter_id: UUID of an existing filter to use for the query
+            sort_criteria: List of sort criteria (dicts that can contain
+                a field, stat and order)
+            data_columns: List of fields to aggregate data from
+            group_column: The field to group the entities by
+            subgroup_column: The field to further group the entities
+                inside groups by
+            text_columns: List of simple text columns which no statistics
+                are calculated for
+            first_group: The index of the first aggregate group to return
+            max_groups: The maximum number of aggregate groups to return,
+                -1 for all
+            mode: Special mode for aggregation
+
+        Returns:
+            The response. See :py:meth:`send_command` for details.
+        """
+        if not resource_type:
+            raise RequiredArgument(
+                function=self.get_aggregates.__name__, argument='resource_type'
+            )
+
+        if not isinstance(resource_type, self.types.EntityType):
+            raise InvalidArgumentType(
+                function=self.get_aggregates.__name__,
+                argument='resource_type',
+                arg_type=self.types.EntityType.__name__,
+            )
+
+        cmd = XmlCommand('get_aggregates')
+
+        _actual_resource_type = resource_type
+        if resource_type.value == EntityType.AUDIT.value:
+            _actual_resource_type = EntityType.TASK
+            cmd.set_attribute('usage_type', 'audit')
+        elif resource_type.value == EntityType.POLICY.value:
+            _actual_resource_type = EntityType.SCAN_CONFIG
+            cmd.set_attribute('usage_type', 'policy')
+        elif resource_type.value == EntityType.SCAN_CONFIG.value:
+            cmd.set_attribute('usage_type', 'scan')
+        elif resource_type.value == EntityType.TASK.value:
+            cmd.set_attribute('usage_type', 'scan')
+        cmd.set_attribute('type', _actual_resource_type.value)
+
+        _add_filter(cmd, filter, filter_id)
+
+        if first_group is not None:
+            if not isinstance(first_group, int):
+                raise InvalidArgumentType(
+                    function=self.get_aggregates.__name__,
+                    argument='first_group',
+                    arg_type=int.__name__,
+                )
+            cmd.set_attribute('first_group', str(first_group))
+
+        if max_groups is not None:
+            if not isinstance(max_groups, int):
+                raise InvalidArgumentType(
+                    function=self.get_aggregates.__name__,
+                    argument='max_groups',
+                    arg_type=int.__name__,
+                )
+            cmd.set_attribute('max_groups', str(max_groups))
+
+        if sort_criteria is not None:
+            if not isinstance(sort_criteria, list):
+                raise InvalidArgumentType(
+                    function=self.get_aggregates.__name__,
+                    argument='sort_criteria',
+                    arg_type=list.__name__,
+                )
+            for sort in sort_criteria:
+                if not isinstance(sort, dict):
+                    raise InvalidArgumentType(
+                        function=self.get_aggregates.__name__,
+                        argument='sort_criteria',
+                    )
+
+                sort_elem = cmd.add_element('sort')
+                if sort.get('field'):
+                    sort_elem.set_attribute('field', sort.get('field'))
+
+                if sort.get('stat'):
+                    if isinstance(sort['stat'], AggregateStatistic):
+                        sort_elem.set_attribute('stat', sort['stat'].value)
+                    else:
+                        stat = get_aggregate_statistic_from_string(sort['stat'])
+                        sort_elem.set_attribute('stat', stat.value)
+
+                if sort.get('order'):
+                    if isinstance(sort['order'], SortOrder):
+                        sort_elem.set_attribute('order', sort['order'].value)
+                    else:
+                        so = get_sort_order_from_string(sort['order'])
+                        sort_elem.set_attribute('order', so.value)
+
+        if data_columns is not None:
+            if not isinstance(data_columns, list):
+                raise InvalidArgumentType(
+                    function=self.get_aggregates.__name__,
+                    argument='data_columns',
+                    arg_type=list.__name__,
+                )
+            for column in data_columns:
+                cmd.add_element('data_column', column)
+
+        if group_column is not None:
+            cmd.set_attribute('group_column', group_column)
+
+        if subgroup_column is not None:
+            if not group_column:
+                raise RequiredArgument(
+                    '{} requires a group_column argument'
+                    ' if subgroup_column is given'.format(
+                        self.get_aggregates.__name__
+                    ),
+                    function=self.get_aggregates.__name__,
+                    argument='subgroup_column',
+                )
+            cmd.set_attribute('subgroup_column', subgroup_column)
+
+        if text_columns is not None:
+            if not isinstance(text_columns, list):
+                raise InvalidArgumentType(
+                    function=self.get_aggregates.__name__,
+                    argument='text_columns',
+                    arg_type=list.__name__,
+                )
+            for column in text_columns:
+                cmd.add_element('text_column', column)
+
+        if mode is not None:
+            cmd.set_attribute('mode', mode)
+
+        # Add additional keyword args as attributes for backward compatibility.
+        cmd.set_attributes(kwargs)
+
+        return self._send_xml_command(cmd)
+
     def get_tls_certificates(
         self,
         *,
@@ -711,6 +1044,96 @@ class GmpV9Mixin(GvmProtocol):
             preferences=preferences,
         )
 
+    def modify_permission(
+        self,
+        permission_id: str,
+        *,
+        comment: Optional[str] = None,
+        name: Optional[str] = None,
+        resource_id: Optional[str] = None,
+        resource_type: Optional[EntityType] = None,
+        subject_id: Optional[str] = None,
+        subject_type: Optional[PermissionSubjectType] = None
+    ) -> Any:
+        """Modifies an existing permission.
+
+        Arguments:
+            permission_id: UUID of permission to be modified.
+            comment: The comment on the permission.
+            name: Permission name, currently the name of a command.
+            subject_id: UUID of subject to whom the permission is granted
+            subject_type: Type of the subject user, group or role
+            resource_id: UUID of entity to which the permission applies
+            resource_type: Type of the resource. For Super permissions user,
+                group or role
+
+        Returns:
+            The response. See :py:meth:`send_command` for details.
+        """
+        if not permission_id:
+            raise RequiredArgument(
+                function=self.modify_permission.__name__,
+                argument='permission_id',
+            )
+
+        cmd = XmlCommand("modify_permission")
+        cmd.set_attribute("permission_id", permission_id)
+
+        if comment:
+            cmd.add_element("comment", comment)
+
+        if name:
+            cmd.add_element("name", name)
+
+        if resource_id or resource_type:
+            if not resource_id:
+                raise RequiredArgument(
+                    function=self.modify_permission.__name__,
+                    argument='resource_id',
+                )
+
+            if not resource_type:
+                raise RequiredArgument(
+                    function=self.modify_permission.__name__,
+                    argument='resource_type',
+                )
+
+            if not isinstance(resource_type, self.types.EntityType):
+                raise InvalidArgumentType(
+                    function=self.modify_permission.__name__,
+                    argument='resource_type',
+                    arg_type=self.types.EntityType.__name__,
+                )
+
+            _xmlresource = cmd.add_element(
+                "resource", attrs={"id": resource_id}
+            )
+            _actual_resource_type = resource_type
+            if resource_type.value == EntityType.AUDIT.value:
+                _actual_resource_type = EntityType.TASK
+            elif resource_type.value == EntityType.POLICY.value:
+                _actual_resource_type = EntityType.SCAN_CONFIG
+            _xmlresource.add_element("type", _actual_resource_type.value)
+
+        if subject_id or subject_type:
+            if not subject_id:
+                raise RequiredArgument(
+                    function=self.modify_permission.__name__,
+                    argument='subject_id',
+                )
+
+            if not isinstance(subject_type, PermissionSubjectType):
+                raise InvalidArgumentType(
+                    function=self.modify_permission.__name__,
+                    argument='subject_type',
+                    arg_type=PermissionSubjectType.__name__,
+                )
+
+            _xmlsubject = cmd.add_element("subject", attrs={"id": subject_id})
+            _xmlsubject.add_element("type", subject_type.value)
+
+        return self._send_xml_command(cmd)
+
     def modify_policy_set_nvt_preference(
         self,
         policy_id: str,
@@ -801,29 +1224,113 @@ class GmpV9Mixin(GvmProtocol):
     def modify_policy_set_family_selection(
         self,
         policy_id: str,
-        families: List[str],
+        families: List[Tuple[str, bool]],
         *,
-        auto_add_new_families: Optional[bool] = True,
-        auto_add_new_nvts: Optional[bool] = True
+        auto_add_new_families: Optional[bool] = True
     ) -> Any:
         """
         Selected the NVTs of a policy at a family level.
 
         Arguments:
             policy_id: UUID of policy to modify.
-            families: List of NVT family names to select.
+            families: A list of tuples with the first entry being the name
+                of the NVT family selected, second entry a boolean indicating
+                whether new NVTs should be added to the family automatically.
             auto_add_new_families: Whether new families should be added to the
                 policy automatically. Default: True.
-            auto_add_new_nvts: Whether new NVTs in the selected families should
-                be added to the policy automatically. Default: True.
         """
         self.modify_config_set_family_selection(
             config_id=policy_id,
             families=families,
             auto_add_new_families=auto_add_new_families,
-            auto_add_new_nvts=auto_add_new_nvts,
         )
 
+    def modify_tag(
+        self,
+        tag_id: str,
+        *,
+        comment: Optional[str] = None,
+        name: Optional[str] = None,
+        value=None,
+        active=None,
+        resource_action: Optional[str] = None,
+        resource_type: Optional[EntityType] = None,
+        resource_filter: Optional[str] = None,
+        resource_ids: Optional[List[str]] = None
+    ) -> Any:
+        """Modifies an existing tag.
+
+        Arguments:
+            tag_id: UUID of the tag.
+            comment: Comment to add to the tag.
+            name: Name of the tag.
+            value: Value of the tag.
+            active: Whether the tag is active.
+            resource_action: Whether to add or remove resources instead of
+                overwriting. One of '', 'add', 'set' or 'remove'.
+            resource_type: Type of the resources to which to attach the tag.
+                Required if resource_filter is set.
+            resource_filter: Filter term to select resources the tag is to be
+                attached to.
+            resource_ids: IDs of the resources to which to attach the tag.
+
+        Returns:
+            The response. See :py:meth:`send_command` for details.
+        """
+        if not tag_id:
+            raise RequiredArgument(
+                function=self.modify_tag.__name__, argument='tag_id'
+            )
+
+        cmd = XmlCommand("modify_tag")
+        cmd.set_attribute("tag_id", str(tag_id))
+
+        if comment:
+            cmd.add_element("comment", comment)
+
+        if name:
+            cmd.add_element("name", name)
+
+        if value:
+            cmd.add_element("value", value)
+
+        if active is not None:
+            cmd.add_element("active", _to_bool(active))
+
+        if resource_action or resource_filter or resource_ids or resource_type:
+            if resource_filter and not resource_type:
+                raise RequiredArgument(
+                    function=self.modify_tag.__name__, argument='resource_type'
+                )
+
+            _xmlresources = cmd.add_element("resources")
+            if resource_action is not None:
+                _xmlresources.set_attribute("action", resource_action)
+
+            if resource_filter is not None:
+                _xmlresources.set_attribute("filter", resource_filter)
+
+            for resource_id in resource_ids or []:
+                _xmlresources.add_element(
+                    "resource", attrs={"id": str(resource_id)}
+                )
+
+            if resource_type is not None:
+                if not isinstance(resource_type, self.types.EntityType):
+                    raise InvalidArgumentType(
+                        function=self.modify_tag.__name__,
+                        argument="resource_type",
+                        arg_type=EntityType.__name__,
+                    )
+                _actual_resource_type = resource_type
+                if resource_type.value == EntityType.AUDIT.value:
+                    _actual_resource_type = EntityType.TASK
+                elif resource_type.value == EntityType.POLICY.value:
+                    _actual_resource_type = EntityType.SCAN_CONFIG
+                _xmlresources.add_element("type", _actual_resource_type.value)
+
+        return self._send_xml_command(cmd)
+
     def modify_tls_certificate(
         self,
         tls_certificate_id: str,
